@@ -1,13 +1,15 @@
 from __future__ import annotations
 
 import cmath
+import enum
 from logging import getLogger
-from typing import Dict, List, Tuple
 
 import numpy as np
+import psutil
+import ray
 
-from pymwm.utils.slit_utils import ABY_cython, coefs_cython, uvABY_cython
-from pymwm.waveguide import Database, Waveguide
+from pymwm.utils import slit_utils
+from pymwm.waveguide import Database, Sampling, Waveguide
 
 from .samples import Samples, SamplesForRay, SamplesLowLoss, SamplesLowLossForRay
 
@@ -60,8 +62,8 @@ class Slit(Waveguide):
             params["modes"]["num_m"] = 1
         super().__init__(params)
 
-    def get_alphas(self, alpha_list: List[Tuple[str, int, int]]) -> Dict:
-        alphas = {"h": [], "v": []}
+    def get_alphas(self, alpha_list: list[tuple[str, int, int]]) -> dict:
+        alphas: dict = {"h": [], "v": []}
         for alpha in [("E", n, 1) for n in range(1, self.num_n)]:
             if alpha in alpha_list:
                 alphas["v"].append(alpha)
@@ -87,15 +89,14 @@ class Slit(Waveguide):
             success_list.append(success_e[:, :, -1])
         return np.dstack(xs_list), np.dstack(success_list)
 
-    def betas_convs_samples(
-        self, params: Dict
-    ) -> Tuple[np.ndarray, np.ndarray, Samples]:
+    def betas_convs_samples(self, params: dict) -> tuple[dict, dict, Sampling]:
         im_factor = self.clad.im_factor
         self.clad.im_factor = 1.0
         self.clad_params["im_factor"] = 1.0
         p_modes = params["modes"].copy()
         num_n_0 = p_modes["num_n"]
-        betas = convs = None
+        betas: dict = {}
+        convs: dict = {}
         success = False
         catalog = Database().load_catalog()
         num_n_max = catalog["num_n"].max()
@@ -110,84 +111,108 @@ class Slit(Waveguide):
                 except IndexError:
                     continue
         if not success:
-            import ray
-
             p_modes["num_n"] = num_n_0
-            smp = Samples(self.r, self.fill_params, self.clad_params, p_modes)
+            betas, convs, smp = self.do_sampling(p_modes)
+        if im_factor != 1.0:
+            self.clad.im_factor = im_factor
+            self.clad_params["im_factor"] = im_factor
+            betas, convs, smp = self.do_sampling_for_im_factor(betas, convs, p_modes)
+        return betas, convs, smp
+
+    def do_sampling(self, p_modes: dict) -> tuple[dict, dict, Samples]:
+        num_n_0 = p_modes["num_n"]
+        smp = Samples(self.r, self.fill_params, self.clad_params, p_modes)
+        ray.shutdown()
+        try:
+            ray.init()
+            p_modes_id = ray.put(p_modes)
+            pool = ray.util.ActorPool(
+                SamplesForRay.remote(
+                    self.r, self.fill_params, self.clad_params, p_modes_id
+                )
+                for _ in range(psutil.cpu_count())
+            )
+            args0 = [
+                ("M", "even", num_n_0),
+                ("M", "odd", num_n_0),
+                ("E", "even", num_n_0),
+                ("E", "odd", num_n_0),
+            ]
+            xs_success_wr_list = list(
+                pool.map(lambda a, arg: a.wr_sampling.remote(arg), args0)
+            )
+            num_wr = xs_success_wr_list[0][0].shape[0]
+            args1 = []
+            for i, (pol, parity, _) in enumerate(args0):
+                xs_array, _ = xs_success_wr_list[i]
+                for iwr in range(num_wr):
+                    args1.append((pol, parity, num_n_0, iwr, xs_array[iwr]))
+            xs_success_wi_list: list[tuple[np.ndarray, np.ndarray]] = list(
+                pool.map(lambda a, arg: a.wi_sampling.remote(arg), args1)
+            )
+        finally:
+            ray.shutdown()
+        num_wi = xs_success_wi_list[0][0].shape[0]
+        xs_success_list: list[tuple[np.ndarray, np.ndarray]] = []
+        for j, (pol, parity, _) in enumerate(args0):
+            num = xs_success_wi_list[j][0].shape[1]
+            xs_array = np.zeros((num_wr, num_wi, num), dtype=complex)
+            success_array = np.zeros((num_wr, num_wi, num), dtype=bool)
+            for iwr in range(num_wr):
+                i = j * num_wr + iwr
+                xs_i, success_i = xs_success_wi_list[i]
+                xs_array[iwr] = xs_i
+                success_array[iwr] = success_i
+            xs_success_list.append((xs_array, success_array))
+        xs_success_M = self.merge_even_and_odd_data(
+            xs_success_list[0], xs_success_list[1]
+        )
+        xs_success_E = self.merge_even_and_odd_data(
+            xs_success_list[2], xs_success_list[3]
+        )
+        betas, convs = smp.betas_convs([xs_success_M, xs_success_E])
+        smp.database.save(betas, convs)
+        return betas, convs, smp
+
+    def do_sampling_for_im_factor(
+        self, betas: dict, convs: dict, p_modes: dict
+    ) -> tuple[dict, dict, SamplesLowLoss]:
+        smp = SamplesLowLoss(self.r, self.fill_params, self.clad_params, p_modes)
+        try:
+            betas, convs = smp.database.load()
+        except IndexError:
+            num_n = p_modes["num_n"]
+            ns = list(range(num_n))
+            ns_e = ns[::2]
+            ns_o = ns[1::2]
+            args = []
+            for iwr in range(len(smp.ws)):
+                for iwi in range(len(smp.wis)):
+                    xis_list = [
+                        [betas[("M", n, 1)][iwr, iwi] ** 2 for n in ns_e],
+                        [betas[("M", n, 1)][iwr, iwi] ** 2 for n in ns_o],
+                        [betas[("E", n, 1)][iwr, iwi] ** 2 for n in ns_e],
+                        [betas[("E", n, 1)][iwr, iwi] ** 2 for n in ns_o],
+                    ]
+                    args.append((iwr, iwi, xis_list))
             ray.shutdown()
             try:
                 ray.init()
                 p_modes_id = ray.put(p_modes)
                 pool = ray.util.ActorPool(
-                    SamplesForRay.remote(
+                    SamplesLowLossForRay.remote(
                         self.r, self.fill_params, self.clad_params, p_modes_id
                     )
-                    for _ in range(4)
+                    for _ in range(psutil.cpu_count())
                 )
                 xs_success_list = list(
-                    pool.map(
-                        lambda a, arg: a.task.remote(arg),
-                        [
-                            ("M", "even", num_n_0),
-                            ("M", "odd", num_n_0),
-                            ("E", "even", num_n_0),
-                            ("E", "odd", num_n_0),
-                        ],
-                    )
+                    pool.map(lambda a, arg: a.task.remote(arg), args)
                 )
+
             finally:
                 ray.shutdown()
-            xs_success_M = self.merge_even_and_odd_data(
-                xs_success_list[0], xs_success_list[1]
-            )
-            xs_success_E = self.merge_even_and_odd_data(
-                xs_success_list[2], xs_success_list[3]
-            )
-            betas, convs = smp.betas_convs([xs_success_M, xs_success_E])
+            betas, convs = smp.betas_convs(xs_success_list)
             smp.database.save(betas, convs)
-        if im_factor != 1.0:
-            self.clad.im_factor = im_factor
-            self.clad_params["im_factor"] = im_factor
-            smp = SamplesLowLoss(self.r, self.fill_params, self.clad_params, p_modes)
-            try:
-                betas, convs = smp.database.load()
-            except IndexError:
-                self.clad.im_factor = im_factor
-                self.clad_params["im_factor"] = im_factor
-                num_n = p_modes["num_n"]
-                ns = list(range(num_n))
-                ns_e = ns[::2]
-                ns_o = ns[1::2]
-                args = []
-                for iwr in range(len(smp.ws)):
-                    for iwi in range(len(smp.wis)):
-                        xis_list = [
-                            [betas[("M", n, 1)][iwr, iwi] ** 2 for n in ns_e],
-                            [betas[("M", n, 1)][iwr, iwi] ** 2 for n in ns_o],
-                            [betas[("E", n, 1)][iwr, iwi] ** 2 for n in ns_e],
-                            [betas[("E", n, 1)][iwr, iwi] ** 2 for n in ns_o],
-                        ]
-                        args.append((iwr, iwi, xis_list))
-                import ray
-
-                ray.shutdown()
-                try:
-                    ray.init()
-                    p_modes_id = ray.put(p_modes)
-                    pool = ray.util.ActorPool(
-                        SamplesLowLossForRay.remote(
-                            self.r, self.fill_params, self.clad_params, p_modes_id
-                        )
-                        for _ in range(16)
-                    )
-                    xs_success_list = list(
-                        pool.map(lambda a, arg: a.task.remote(arg), args)
-                    )
-
-                finally:
-                    ray.shutdown()
-                betas, convs = smp.betas_convs(xs_success_list)
-                smp.database.save(betas, convs)
         return betas, convs, smp
 
     def beta(self, w, alpha):
@@ -678,7 +703,7 @@ class Slit(Waveguide):
         return np.ascontiguousarray(As), np.ascontiguousarray(Bs)
 
     def coefs(self, hs, w):
-        return coefs_cython(self, hs, w)
+        return slit_utils.coefs_cython(self, hs, w)
 
     def Ys(self, w, hs, As, Bs):
         vals = []
@@ -695,18 +720,20 @@ class Slit(Waveguide):
     def ABY(self, w, hs):
         e1 = self.fill(w)
         e2 = self.clad(w)
-        return ABY_cython(w, self.r, self.s_all, self.n_all, hs, e1, e2)
+        return slit_utils.ABY_cython(w, self.r, self.s_all, self.n_all, hs, e1, e2)
 
     def hABY(self, w):
         e1 = self.fill(w)
         e2 = self.clad(w)
         hs = np.array([self.beta(w, alpha) for alpha in self.alpha_all])
-        As, Bs, Y = ABY_cython(w, self.r, self.s_all, self.n_all, hs, e1, e2)
+        As, Bs, Y = slit_utils.ABY_cython(w, self.r, self.s_all, self.n_all, hs, e1, e2)
         return hs, As, Bs, Y
 
     def huvABY(self, w):
         e1 = self.fill(w)
         e2 = self.clad(w)
         hs = np.array([self.beta(w, alpha) for alpha in self.alpha_all])
-        us, vs, As, Bs, Y = uvABY_cython(w, self.r, self.s_all, self.n_all, hs, e1, e2)
+        us, vs, As, Bs, Y = slit_utils.uvABY_cython(
+            w, self.r, self.s_all, self.n_all, hs, e1, e2
+        )
         return hs, us, vs, As, Bs, Y
